//
//  YKPrintManager.h
//  YKPrinterKit
//
//  Created by YK on 2022/5/9.
//

#import <Foundation/Foundation.h>
#import <YKPrinterKit/YKPrinter.h>

NS_ASSUME_NONNULL_BEGIN

@interface YKPrintManager : NSObject

/** 可连接的设备列表 */
@property (nonatomic, strong, readonly) NSMutableArray<YKPrinter *>*printerArray;

/** 当前连接的设备对象 */
@property (nonatomic, strong, readonly) YKPrinter *connectedPrinter;

/** 当前设备是否可以打印 */
@property (nonatomic, assign, readonly) BOOL isCanPrint;

/** 信号强度 搜索到的信号值小于这个就不会出现在设备列表中，默认-100 */
@property (nonatomic, assign) NSInteger validRssi;

#pragma mark - 初始化
+ (instancetype)sharedInstance;


#pragma mark - 蓝牙连接与断开
/**
 * 搜索打印设备:每次扫描都会发送一次蓝牙状态通知，如果蓝牙不是可链接状态会立即回调并且不会进行扫描
 * @param ptTypes 目标设备类型列表（数组元素是YKPrinterType）
 * @param second 搜索时间(单位:s, s<=0, 表示时间不限)
 * @param complete 扫描停止后回调
 */
- (void)scanForPrinter:(NSArray *)ptTypes duration:(NSInteger)second scanEnd:(void(^_Nullable)(void))complete;

/**
 * 连接到打印设备
 * @param printer 打印设备
 */
- (void)connectPrinter:(YKPrinter *)printer;

/**
 * 断开打印设备
 */
- (void)disconnectPrinter;

/**
 * 停止扫描
 */
- (void)stopScaning;

#pragma mark - 指令
/**
 * 设置自动关机时间[1-60]单位10min, 0表示不自动关机。
 */
- (void)setPrinterAutoOffTime:(NSUInteger)offtime;


/**
 * 空白走纸
 */
- (void)printWhiteSpace:(NSInteger)length;


/**
 * 设置TP4机器当前纸张尺寸，内部会修改当前printer的widthDot（TP4机器特有，其他型号机器不要使用）
 */
- (void)setPrinterPaperSize:(YKTP4PaperSize)size;


/**
 * 获取纸张信息,LP2-S机器特有，其他型号不要使用
 */
- (void)lp2PaperInfo:(void(^)(NSDictionary *dict))complete;

/**
 * 获取固件版本号
 */
- (void)printerFirmwareInfo;

/**
 * 获取耗材信息(只适用于t81机器)
 */
- (void)checkConsumablesInfo:(void(^)(NSString *infoStr))complete;

/**
 * 打印图片
 * @param image 目标图片
 * @param idensity 打印浓度
 * @param count 打印份数
 * @param isLabel 是否标签纸
 * @param isWhite 是否空白走纸
*/
- (void)printImage:(UIImage *)image
           density:(YKPrinterDensity)idensity
             count:(NSInteger)count
        labelState:(BOOL)isLabel
        whitespace:(BOOL)isWhite;

/**
 * 打印图组
 * @param imageArr 目标图片组
 * @param idensity 打印浓度
 * @param count 打印份数
 * @param isLabel 是否标签纸
 * @param isWhite 是否空白走纸
 */
- (void)printImageArry:(NSArray <UIImage *> *)imageArr
               density:(YKPrinterDensity)idensity
                 count:(NSInteger)count
            labelState:(BOOL)isLabel
            whitespace:(BOOL)isWhite;


/**
 *  LP1-S 标签纸打印
 * @param image     目标图片
 * @param density 打印浓度
 * @param width     标签纸宽度 单位：mm 例如15*50mm width传50
 * @param count     打印份数
 */
- (void)labelPrintImage:(UIImage *)image
                density:(YKPrinterDensity)density
             paperWidth:(NSInteger)width
             printCount:(NSInteger)count;


/**
 *  LP2-S 标签纸打印
 * @param image     目标图片
 * @param density 打印浓度
 * @param width      标签纸宽度 单位：mm 例如15*50mm width传50
 * @param count     打印份数
 */
- (void)printLabelImageWithFeedback:(UIImage *)image
                            density:(YKPrinterDensity)density
                         paperWidth:(NSInteger)width
                         printCount:(NSInteger)count;


/**
 * LP2-S 连续纸打印
 * @param image 目标图片
 * @param density 打印浓度
 * @param count     打印份数
 */
- (void)lp2ContinuousPrintImage:(UIImage *)image
                        density:(YKPrinterDensity)density
                     printCount:(NSInteger)count;


/**
 * TP6 标签纸打印
 * @param image        需要打印的图
 * @param density    打印浓度
 * @param width        打印纸张宽度
 * @param count       打印份数
 */
- (void)printLabelImageForTP6:(UIImage *)image
                      density:(YKPrinterDensity)density
                   labelWidth:(int)width
                   printCount:(NSInteger)count;

/**
 * 固件升级
 * @param firmwardData 固件文件
 * @param callback 升级进度回调
 * @param block 是否升级成功回调
 */
- (void)printerUpateFirmware:(NSData *)firmwardData
                  processing:(void(^)(CGFloat percent))callback
                    complete:(void (^)(BOOL bSuccess))block;



#pragma mark - SDK版本等相关方法
- (NSString *)getVersion;


#pragma mark - 其他
/**
 * 获取上一次连接过的printer的mac地址
 */
- (NSString *)lastConnectedPrinterMAC;

/**
 * 清空设备列表
 */
- (void)clearPrinterList;

/**
 *  根据打印机类型获取打印机打印宽度 当type有误时，会默认返回300dpi的宽度
 *  @param type 打印机类型
 *  @return 打印宽度
 */
- (YKPrinterWidthDot)getPrinterWidthDotWithType:(YKPrinterType)type;

/**
 * D81设备使用
 * 图片转换成斑马指令
 * 下面两个获取data的方法里面会强制转成斑马指令
 */
+ (NSData *)imageToZebraInstruction:(UIImage *)targetImg printSpeed:(int)speed printDensity:(int)density autoCout:(BOOL)isAuto;

/**
 * 设置指令类型
 * 指令类型分两种：1.小机器，边发边打 2.a4机器 斑马指令
 * @param isCus  YES-小机器   NO-斑马指令
 * */
- (void)setCodeMethod:(BOOL)isCus;
@end

#pragma mark - 通知名
/** 扫描到新设备后会触发这个通知 */
extern NSString * const YK_FOUND_NEW_PRINTER_NOTIFICATION_NAME;

/** 打印机连接状态发生改变后会触发这个通知 */
extern NSString * const YK_PRINTER_CONNECT_STATE_CHANGE_NOTIFICATION_NAME;

/** 打印机会主动上报状态 5s/次，如果有异常或主动查询会立即上报，每上报一次都会触发一次这个通知 */
extern NSString * const YK_SYNC_PRINTER_STATE_NOTIFICATION_NAME;

/** 当手机蓝牙状态发生改变，会触发这个通知 */
extern NSString * const YK_BLUETOOTH_STATE_CHANGE_NOTIFICATION_NAME;

/** 当打印机打印状态发生变化时会触发这个通知 */
extern NSString * const YK_UPDATE_PRINT_STATE_NOTIFICATION_NAME;

NS_ASSUME_NONNULL_END
